@@ -1,10 +1,10 @@
 package Finance::Bank::DE::NetBank;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION $DEBUG);
 use base qw(Class::Accessor);
 Finance::Bank::DE::NetBank->mk_accessors(
-    qw(BASE_URL BLZ CUSTOMER_ID PASSWORD AGENT_TYPE AGENT ACCOUNT));
+    qw(BASE_URL BLZ CUSTOMER_ID PASSWORD AGENT_TYPE AGENT ACCOUNT Debug));
 
 use WWW::Mechanize;
 use HTML::TreeBuilder;
@@ -13,7 +13,7 @@ use Data::Dumper;
 
 $| = 1;
 
-$VERSION = "1.00";
+$VERSION = "1.04_01";
 
 sub Version {
     return $VERSION;
@@ -22,20 +22,16 @@ sub Version {
 sub new {
     my $proto  = shift;
     my %values = (
-        BASE_URL =>
-          "https://www.netbank-money.de/netbank-barrierefrei-banking/view/",
-        BLZ         => "20090500",    # NetBank BLZ
-        CUSTOMER_ID => "demo",        # Demo Login
-        PASSWORD    => "",            # Demo does not require a password
-        ACCOUNT     => "1234567",     # Demo Account Number (Kontonummer)
         AGENT_TYPE => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1) ",
+        BASE_URL =>
+          'https://www.netbank-money.de/netbank-barrierefrei-banking/view/',
+        BLZ         => '20090500', # NetBank BLZ
+        CUSTOMER_ID => '',
+        PASSWORD    => '',
+        ACCOUNT     => '', 
         @_
     );
 
-    if ( $values{'CUSTOMER_ID'} ne "demo" && $values{'ACCOUNT'} eq "2777770" ) {
-        $values{'ACCOUNT'} = $values{'CUSTOMER_ID'};
-    }
-
     my $class = ref($proto) || $proto;
     my $parent = ref($proto) && $proto;
 
@@ -51,10 +47,8 @@ sub new {
 
 sub connect {
     my $self  = shift;
-    my $url   = $self->BASE_URL() . "index.jsp?blz=" . $self->BLZ();
-    my $agent = WWW::Mechanize->new( agent => $self->AGENT_TYPE(), );
-    $agent->get($url);
-    $self->AGENT($agent);
+    print STDERR "Method connect() is deprecated. Use only login() instead!\n";
+    return $self->login(@_);
 }
 
 sub login {
@@ -64,17 +58,36 @@ sub login {
         PASSWORD    => $self->PASSWORD(),
         @_
     );
+   
+    my $url   = $self->BASE_URL() . "index.jsp?blz=" . $self->BLZ();
+    my $agent = WWW::Mechanize->new( agent => $self->AGENT_TYPE(), );
+    $agent->get($url);
+    $self->AGENT($agent);
 
-    my $agent = $self->AGENT();
     $agent->field( "kundennummer", $values{'CUSTOMER_ID'} );
     $agent->field( "pin",          $values{'PASSWORD'} );
     $agent->click();
+
+    print STDERR Dumper( $agent->content ) if $self->Debug();
+    
+    if ($agent->content =~ /fieldtableerrorred/ig) {
+        return undef;
+    }
+
+    return 1;
+ 
 }
 
 sub saldo {
     my $self = shift;
     my $data = $self->statement(@_);
-    return $data->{'STATEMENT'}{'SALDO'};
+   
+    if ($data) { 
+        print STDERR Dumper($data) if $self->Debug();
+        return $data->{'STATEMENT'}{'SALDO'};
+    } else {
+        return undef;
+    }
 }
 
 sub statement {
@@ -88,6 +101,10 @@ sub statement {
         @_
     );
 
+    # get mainpage
+    my $login_status = $self->login();
+    return undef unless $login_status;
+       
     my $agent = $self->AGENT();
 
 #   If you've problems with your environmet settings activate this and "use encodings"
@@ -106,7 +123,10 @@ sub statement {
 
     $agent->click();
     $agent->get( $self->BASE_URL() . "/umsatzdownload.do" );
-    my $content     = $agent->content();
+
+    my $content = $agent->content();
+    print STDERR Dumper($content) if $self->Debug();
+
     my $csv_content = $self->_parse_csv($content);
     return $csv_content;
 }
@@ -125,6 +145,10 @@ sub transfer {
         TAN              => "",
         @_
     );
+    
+    # get mainpage
+    my $login_status = $self->login();
+    return undef unless $login_status;
 
     my $agent = $self->AGENT();
     my $url   = $self->BASE_URL();
@@ -133,26 +157,19 @@ sub transfer {
 
     ( $values{'AMOUNT_EURO'}, $values{'AMOUNT_CENT'} ) =
       split( /\.|,/, $values{'AMOUNT'} );
+    
     $values{'AMOUNT_CENT'} = sprintf( "%02d", $values{'AMOUNT_CENT'} );
-
     $agent->field( "auftraggeberKontonummer", $values{'SENDER_ACCOUNT'} );
-
     $agent->field( "empfaengerName", $values{'RECEIVER_NAME'} );
-
-#   $agent->field("empfaengerDatenSpeichern", $values{'RECEIVER_SAVE'}); # waiting for WWW::Mechanize support
     $agent->field( "empfaengerBankleitzahl", $values{'RECEIVER_BLZ'} );
     $agent->field( "empfaengerKontonummer",  $values{'RECEIVER_ACCOUNT'} );
-
     $agent->field( "betragEuro", $values{'AMOUNT_EURO'} );
     $agent->field( "betragCent", $values{'AMOUNT_CENT'} );
-
     $agent->field( "verwendungszweck1", $values{'COMMENT_1'} );
     $agent->field( "verwendungszweck2", $values{'COMMENT_2'} );
-
     $agent->click("btnNeuSpeichern");
 
     # sure it's right ...
-
     $agent->field( "ubo", $values{'TAN'} );
     $agent->click("btnBestaetigen");
 
@@ -162,11 +179,10 @@ sub transfer {
         $agent->content() =~ m|<span class="error">(.*?)</span>|;
         my $error = $1;
         print "ERROR: $error";
-        return ( 0, 0 );
-    }
-
-    else {
-        ( 1, $agent->content() );
+        return undef;
+    } else {
+        my $content = $agent->content();
+        return $agent->content();
     }
 
 }
@@ -237,36 +253,35 @@ sub _parse_csv {
 
 1;
 __END__
-# Below is the stub of documentation for your module. You better edit it!
-
 
 =head1 NAME
 
-Finance::Bank::DE::NetBank - Check your SpardaBank Bank Accounts with Perl
+Finance::Bank::DE::NetBank - Check your NetBank Bank Accounts with Perl
 
 =head1 SYNOPSIS
 
- use Finance::Bank::DE::NetBank;
- my $account = Finance::Bank::DE::NetBank->new(
-						 CUSTOMER_ID => "12345678",
-						 ACCOUNT_ID => "12345678",
-						 PASSWORD => "ROUTE66",
-                                                 BLZ => "70090500",
-                                                 );
- $account->connect(); 
- $account->login();
- print $account->saldo();
- $account->logout();
+    use Finance::Bank::DE::NetBank;
+    
+    my $account = Finance::Bank::DE::NetBank->new(
+        CUSTOMER_ID => '12345678',
+        ACCOUNT => '12345678',
+        PASSWORD => 'ROUTE66',
+        BLZ => '70090500',
+    );
+    
+    if ($account->login()) {
+        print $account->saldo();
+        $account->logout();
+    } 
+    else {
+        print 'login failed. manual interaction needed';
+    }
 
 =head1 DESCRIPTION
 
 
 This module provides a very limited interface to the webbased online banking
 interface of the German "NetBank e.G." operated by Sparda-Datenverarbeitung e.G..
-It will only work with German NetBank accounts - e.g. the Austrian Sparda Bank 
-Accounts will not work.
-
-It uses OOD and doesn't export anything.
 
 B<WARNING!> This module is neither offical nor is it tested to be 100% save! 
 Because of the nature of web-robots, B<everything may break from one day to
@@ -278,47 +293,51 @@ the source of this module yourself to reassure yourself that I am not
 doing anything untoward with your banking data. This software is useful
 to me, but is provided under B<NO GUARANTEE>, explicit or implied.
 
-You can find some basic test scripts for manual testing against the demo-banking
-accounts within the directory "test" in the source directry.
+You can find tests in the C< t > subdirectory. See L< TESTS > for more details.
 
 =head1 METHODS
 
-=head2 new(%values) 
+=head2 my $account = Finance::Bank::DE::NetBank->new(%values) 
 
 This constructor will set the default values and/or user provided values for
 connection and authentication.
 
     my $account = Finance::Bank::DE::NetBank->new (
-                  BASE_URL => "https://www.bankingonline.de/sparda-banking/view/",
-                  BLZ => "70090500",        
-                  CUSTOMER_ID => "demo",    
-                  PASSWORD => "",      
-                  ACCOUNT => "2777770",   
-                  AGENT_TYPE => "Internet Explorer 6",
-	      , @_);
+        BASE_URL => "https://www.bankingonline.de/sparda-banking/view/",
+        BLZ => "70090500",        
+        CUSTOMER_ID => "demo",    
+        PASSWORD => "",      
+        ACCOUNT => "2777770",   
+        AGENT_TYPE => "Internet Explorer 6",
+    , @_);
 
 If you don't provide any values the module will automatically use the demo account.
 
 CUSTOMER_ID is your "Kundennummer" and ACCOUNT is the "Kontonummer" 
 (if you have only one account you can skip that)
 
-=head2 connect()
+=head2 $account->Version()
 
-This method will create the user agent and connect to the online banking website.
-Also this (done by WWW::Mechanize) automagically handles the session-id handling.
+returns the module version
 
-    $account->connect();
+=head2 $account->Debug($value)
 
+Provide a true  C< $value > get some Data::Dumper outputs on STDERR.
 
+=head2 $account->connect()
 
-=head2 login(%values)
+deprecated. use only $account->login()
+
+=head2 $account->login(%values)
 
 This method will try to log in with the provided authentication details. If
 nothing is specified the values from the constructor or the defaults will be used.
 
     $account->login(ACCOUNT => "1234");
 
-=head2 saldo(%values)
+Returns C< undef > on error.
+
+=head2 $account->saldo(%values)
 
 This method will return the current account balance called "Saldo".
 The method uses the account number if previously set. 
@@ -327,8 +346,9 @@ You can override/set it:
 
     $account->saldo(ACCOUNT => "5555555");
 
+Returns C< undef > on error.
 
-=head2 statement(%values)
+=head2 $account->statement(%values)
 
 This method will retrieve an account statement (Kontoauszug) and return a hashref.
 
@@ -337,37 +357,36 @@ The value of TIMEFRAME can be "1" (last day only), "30" (last 30 days only), "al
 START_DATE and END_DATE only).
 
     $account->statement(
-                                 TIMEFRAME => "variabel",
-                                 START_DATE => "10.04.2005",
-                                 END_DATE => "02.05.2005",
-			    );
+        TIMEFRAME => "variabel",
+        START_DATE => "10.04.2005",
+        END_DATE => "02.05.2005",
+    );
 
-=head2 transfer()
+Returns C< undef > on error.
 
-=head2 logout()
+=head2 $account->transfer()
 
-This method will just log out the website and it only exists to keep the module logic clean ;-)
+Returns C< undef > on error.
 
-=head1 USAGE
+=head2 $account->logout()
 
- use Finance::Bank::DE::NetBank;
- use Data::Dumper;
+well - every login method should have a logout method
+
+=head1 TESTS
+
+Since version 1.04 C<Finance::Bank::DE::NetBank> comes with a testsuite.
+It's located in the subdirectory C< t > of the distribution.
+
+To run the tests against the live NetBank demo account use this:
+
+    perl Makefile.PL --livetest
+    make test TEST_VERBOSE=1
+
+The default behaviour is not to test against the live website:
+
+    perl Makefile.PL
+    make test
 
- my $account = Finance::Bank::DE::NetBank->new(
-                                                 BLZ => "70090500",
-                                                 CUSTOMER_ID => "xxxxxxx",
-                                                 ACCOUNT => "yyyyyyy",
-                                                 PASSWORD => "zzzzzz",
-                                                 );
- $account->connect();
- $account->login();
- print Dumper($account->statement(
-                                 TIMEFRAME => "variabel",
-                                 START_DATE => "10.04.2005",
-                                 END_DATE => "02.05.2005",
- 				 )
-             );
- $account->logout();
 
 =head1 BUGS
 
@@ -377,37 +396,24 @@ or email the author.
 
 =head1 HISTORY
 
-1.00 Tue Dec 14 00:43:00 2005
-        - changed URL to barrier free version (works without that new captcha)
-        - replaced buggy html scraping of saldo(). saldo() now uses statement() to retrive the value.
-        
-0.02 Sun May 04 15:45:00 2003
-        - documentation fixes
-
-0.01 Sun May 04 03:00:00 2003
-	- original version;
+see file 'Changes'
 
 =head1 THANK YOU
 
- Torsten Mueller (updated URL, saldo() bug reporting)
+Torsten Mueller (updated URL, saldo() bug reporting)
+Sascha Stock (reported bad example in POD)
 
 =head1 AUTHOR
 
- Roland Moriz
- rmoriz@cpan.org
- http://www.perl-freelancer.de/
-
-Disclaimer stolen from Simon Cozens' Finance::Bank::LloydsTSB without asking for permission %-)
+Roland Moriz
+rmoriz@cpan.org
+http://www.perl-freelancer.de/
 
 =head1 COPYRIGHT
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
-The full text of the license can be found in the
-LICENSE file included with this module.
-
-
 =head1 SEE ALSO
 
 Finance::Bank::DE::SpardaBank, WWW::Mechanize, Finance::Bank::LloydsTSB
